﻿// superball_as class for Actionscript 3 - Jim Bumgardner

package {

	import flash.events.Event;
	import flash.display.Sprite;
	import flash.display.DisplayObject;
	import flash.events.MouseEvent;
	
	public class Superball_mc extends Sprite
	{
		public var isDragging;
		public var vx, vy, lmx, lmy;
		
		static public const gravK = 1.0;			// gravity
		static public const dampK = 0.99;       	// lower numbers make more air resistance
		static public const dampCollision = .95; 	// energy retained in collisions
		static public const ballRadius = 25;
		static public const ballWidth = ballRadius*2;
	
		static private var SW,SH;
		static var mcs = [];
		
		public function Superball_mc(itsPar:DisplayObject)
		{
			SW = itsPar.stage.stageWidth;
			SH = itsPar.stage.stageHeight;
			x = ballRadius+Math.random()*(SW-ballWidth);
			y = ballRadius+Math.random()*(SH-ballWidth);
			vx = Math.random()*32-16;
			vy = Math.random()*32-16;
			width = ballRadius*2;
			height = ballRadius*2;
			addEventListener(Event.ENTER_FRAME, this.moveBall);
			addEventListener(MouseEvent.MOUSE_DOWN, this.pressBall);
			addEventListener(MouseEvent.MOUSE_UP, this.releaseBall);
			mcs.push(this);
			buttonMode = true; // use hand cursors
		}
		
		static public function handleCollisions(evt:Event)
		{
			for (var i = mcs.length-1; i >= 1; --i) {
				var x1 = mcs[i].x;
				var y1 = mcs[i].y;
				for (var j = i-1; j >= 0; --j) {
					var dx = mcs[j].x - x1;
					var dy = mcs[j].y - y1;
					var dist = Math.sqrt(dx*dx+dy*dy);
	
					if (dist < ballWidth) 
					{
						// collide - trade energy (mag1,mag2), and travel in opposite direction of collision
						var mag1 = Math.sqrt(mcs[i].vx*mcs[i].vx+mcs[i].vy*mcs[i].vy);
						var mag2 = Math.sqrt(mcs[j].vx*mcs[j].vx+mcs[j].vy*mcs[j].vy);
						mcs[j].vx = (mag1*dx/dist) * dampCollision;
						mcs[j].vy = (mag1*dy/dist) * dampCollision;
						mcs[i].vx = -(mag2*dx/dist) * dampCollision;
						mcs[i].vy = -(mag2*dy/dist) * dampCollision;
					}
				}
			}
		}
		
		function pressBall(evt:Event)
		{
			this.isDragging = true;
			vx = this.vy = 0;
			lmx = parent.mouseX;
			lmy = parent.mouseY;
			startDrag(false);
		}
	
		function releaseBall(evt:Event)
		{
			this.isDragging = false;
			stopDrag();
		}
	
		function moveBall(evt:Event)
		{
			// compute forces on ball
			if (!this.isDragging) {
				vx *= dampK;	// damping due to air resistance
				vy *= dampK;
		
				vy += gravK;	// gravity
				
				// bounce off floor and ceiling
				if (y+ballRadius + vy >= SH ||
					(y-ballRadius) + vy < 0)
				{
					vy *= -dampCollision;
				}
				// bounce off right and left walls
				if ((x-ballRadius) + vx <= 0 ||
					 x+ballRadius + vx >= SW)
				{
					vx *= -dampCollision;
				}
				x += vx;
				y += vy;
			}
			else {
				// Set vx,vy to correspond to mouse movement, enabling user to 'toss' ball
				vx *= dampK;
				vy *= dampK;
				vx = root.mouseX - lmx;
				vy = root.mouseY - lmy;
				lmx = root.mouseX;
				lmy = root.mouseY;
			}
		}
	}

} // end package
